# {{ project_name }} Requirements
# Generated by ABI-Core CLI

# ============================================
# CORE DEPENDENCIES (Minimal)
# ============================================

# Core Framework
fastapi>=0.100.0
uvicorn[standard]>=0.20.0
pydantic>=2.0.0

# HTTP Client
httpx>=0.25.0

# Note: Additional dependencies are managed by individual services
# - Semantic Layer has its own requirements.txt
# - Guardian has its own requirements.txt
# - Agents have their own requirements.txt