# Agent-specific dependencies only
# Common dependencies are provided by smarbuy/abi-image:latest

{% if with_web_interface %}
# Web interface specific
sse-starlette>=1.6.0
{% endif %}

# Agent-specific libraries (add as needed)
# Example: specialized ML libraries, domain-specific packages