# Semantic Layer specific dependencies only
# Common dependencies are provided by smarbuy/abi-image:latest

# Vector database and embeddings
weaviate-client>=4.0.0
sentence-transformers>=2.2.0

# Google AI (if needed)
google-generativeai

# Ollama client
ollama>=0.1.0